#!/usr/bin/perl
# (c) 2002 Paulo Pinto (japh@ip.pt)
#
# v0.1 :)
#
# check_integrity.pl <absolute URL> <md5sig> <ignoreword>
#
# Nagios plugin to check for webpage changes 
#

require 5.003;
BEGIN { $ENV{PATH} = '/bin' }
use LWP::UserAgent;
use Digest::MD5 qw(md5_hex);
use strict;

sub usage;

my $TIMEOUT = 15;

my %ERRORS = ('UNKNOWN', '-1',
		'OK', '0',
		'WARNING', '1',
		'CRITICAL', '2');
my @input;
my @selected;
my $ignore;
my $site = shift;
my $sig = shift;
my $param = shift;
my $to = shift|| &usage(%ERRORS);

my $state = "OK";
my $answer = undef;

# Just in case of problems, let's not hang Nagios
$SIG{'ALRM'} = sub { 
     close(Server);
     select(STDOUT);
     print "No Answer from Client\n";
     exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);

my $ua = LWP::UserAgent->new;
  $ua->agent("MyApp/0.1 ");
  $ua->timeout($to);
  my $req = HTTP::Request->new(GET => $site);
  $req->content('match=www&errors=0');
  my $res = $ua->request($req);

  if ($res->is_success) {
  @input = split /\n/,$res->content;
  $ignore = $param;
    $param =~ s/\,/\|/g;
    if ($param ne "none") {
    @selected = grep { !/$param/ } @input;
    } else {
    my @selected = @input;
    }
    my $actsig = md5_hex(@selected);
    if ($actsig ne $sig) {
            $answer = "CRITICAL: Site WAS MODIFIED !!! \n";
	    $state = "CRITICAL";
             } else {
            $answer = "OK: Site was NOT modified\n";
	    $state = "OK";
            }
     } else {
     if ($res->message =~ "imeout") {
     $answer = "CRITICAL: Service Check Timed Out\n";
     $state = "CRITICAL";
     } else {
     $answer =  "CRITICAL: Server returned code " . $res->code . " - " . $res->message . " !! Possible break in !\n";
     $state = "CRITICAL";
     }
  }

print $answer;
exit $ERRORS{$state};

sub usage
  {
  print "\nMinimum arguments not supplied!\n\n";
  print "Web Page Integrity Check plugin for Nagios\n";
  print "Copyright (c) 2002 Paulo Pinto\n\n";
  print "Usage: $0 <absolute URL> <md5sig> <ignoreword> <timeout>\n\n";
  print "<absoluteURL> = URL to monitor.\n";
  print "<md5sig> = md5 Signature to compare.\n";
  print "<ignoreword> = Comma separated list of match words for line ignoring (i.e., \"doubleclick\") Use \"none\" to disable.\n";
  print "<timeout> = Timeout value. I'd recommend a value of 10. Increase if the webpage is 'heavy' or slow \n\n\n";

  exit $ERRORS{"UNKNOWN"};
  }
